#!/usr/bin/env python
import unittest
import json
from poetri.generate_jwk_keypair import gen_jwk_keypair
from poetri.generate_public_jwk import gen_public

__author__ = 'Alan Viars @aviars'


class TestJWKGeneration(unittest.TestCase):

    def setUp(self):
        self.mykey = "example.com"

    def test_generate_jwk_keypair(self):

        result = gen_jwk_keypair(self.mykey)

        d = json.loads(result)
        keys = ['kty', 'd', 'e', 'use', 'kid', 'alg', 'n', 'q', 'p']

        """Assert expected keys are in output"""
        for j in d:
            self.assertIn(j, keys)

        self.assertEqual(d['kty'], 'RSA')
        self.assertEqual(d['use'], 'sig')
        self.assertEqual(d['alg'], 'RS256')
        self.assertEqual(d['kid'], self.mykey)

    def test_gen_public(self):
        keypair = gen_jwk_keypair(self.mykey)
        result = gen_public(json.loads(keypair))
        keys = ['kty', 'e', 'use', 'kid', 'alg', 'n', 'q', 'p']
        """Assert expected keys are in output"""
        for j in result:
            self.assertIn(j, keys)

        self.assertNotIn('d', result.keys())
        self.assertEqual(result['kty'], 'RSA')
        self.assertEqual(result['use'], 'sig')
        self.assertEqual(result['alg'], 'RS256')
        self.assertEqual(result['kid'], self.mykey)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
import sys
import json
from collections import OrderedDict
from jwkest.jws import JWS
from jwkest.jwk import RSAKey, DeSerializationNotPossible
from jwkest import BadSignature

__author__ = 'Alan Viars @aviars'


def verify_poet(my_jws, my_jwk_dict):

    # load the Signed JWT (JWS)
    my_jws = my_jws.rstrip().lstrip()
    signed_token = JWS(my_jws)

    # load the JWK
    rsak = RSAKey(**my_jwk_dict)
    try:
        vt = signed_token.verify_compact(signed_token.msg, keys=[rsak])
        retval = vt
    except BadSignature:
        retval = {"error": "The signature did not match"}
    except:
        retval = {"error": str(sys.exc_info())}
    return retval

# command line app.
if __name__ == "__main__":

    if len(sys.argv) != 3:
        print("Usage:")
        print("verify_jws_with_jwk.py [JWT_FILE_PATH] [JWK_PUBLIC_FILE_PATH]")
        print("Example: verify_jws_with_jwk.py my.jwt  public.jwk")
        sys.exit(1)

    my_jwt_file = sys.argv[1]
    my_jwk_file = sys.argv[2]
    jwt_fh = open(my_jwt_file)
    jwk_fh = open(my_jwk_file)

    # Ensure JWK is a JSON object
    try:
        k = jwk_fh.read()
        my_jwk_dict = json.loads(k, object_pairs_hook=OrderedDict)
    except ValueError:
        print("Error parsing the JWK.", str(sys.exc_info()))
        exit(1)

    result = verify_poet(jwt_fh.read(), my_jwk_dict)
    result = json.dumps(result, indent=4)
    jwt_fh.close()
    jwk_fh.close()
    print(result)



#!/usr/bin/env python
from jwkest.jwk import RSAKey
import json
import sys
from collections import OrderedDict

__author__ = 'Alan Viars @aviars'


def gen_public(private_key_jwk):
    rsak = RSAKey(**private_key_jwk)
    return(rsak.serialize())


# Command line app.
if __name__ == "__main__":

    if len(sys.argv) != 2:
        print("Usage:")
        print("generate_public_jwk.py [JWK_KEYPAIR_FILE_PATH]")
        print("generate_public_jwk.py my.jwk")
        sys.exit(1)
    my_jwk_file = sys.argv[1]
    my_jwk_fh = open(my_jwk_file)

    try:
        k = my_jwk_fh.read()
        my_jwk = json.loads(k, object_pairs_hook=OrderedDict)
    except ValueError:
        print("Error parsing the JWK.", str(sys.exc_info()))
        exit(1)

    result = gen_public(my_jwk)
    my_jwk_fh.close()
    print(json.dumps(result, indent=4))



#!/usr/bin/env python
from Crypto.PublicKey import RSA
from jwkest.jwk import RSAKey
import json
import sys

__author__ = 'Alan Viars @aviars'


def gen_jwk_keypair(kid):
    # Mint a new RSA key
    _rsakey = RSA.generate(2048)
    # Wrap it in a JWK class
    _rsajwk = RSAKey(kid=kid, use="sig", alg="RS256", key=_rsakey)
    return json.dumps(_rsajwk.serialize(private=True))

# Command line app.
if __name__ == "__main__":

    if len(sys.argv) != 2:
        print("Usage:")
        print("generate_poet_keypair.py [KID]")
        print("generate_poet_keypair.py example.com")
        sys.exit(1)
    kid = sys.argv[1]
    result = gen_jwk_keypair(kid=kid)
    print(result)



#!/usr/bin/env python
import sys
import json
from jwkest.jws import JWS, NoSuitableSigningKeys
from jwkest.jwt import JWT
from jwkest.jwk import RSAKey, DeSerializationNotPossible
from jwkest import BadSignature
import requests

__author__ = 'Alan Viars @aviars'


def verify_poet_via_url(my_jws):

    # load the Signed JWT (JWS)
    my_jws = my_jws.rstrip().lstrip()
    signed_token = JWS(my_jws)

    # create aplain old JWT so we can fish out the 'iss'
    t = JWT()
    unpacked = t.unpack(token=my_jws)
    payload = unpacked.payload()
    if "iss" not in payload:
        print("Missing 'iss' claim in the JWS payload.")
        exit(1)
    else:
        iss = payload['iss']

    # Fetch the public key from iss

    url = "https://%s/.well-known/poet.jwk" % (iss)
    r = requests.get(url)

    if r.status_code != 200:
        print("The key could not be fetched.")
        exit(1)

    # load the JWK into an RSA Key structure
    rsak = RSAKey(**r.json())
    try:
        vt = signed_token.verify_compact(signed_token.msg, keys=[rsak])
        retval = vt
    except BadSignature:
        retval = {"error": "The signature did not match"}
    except NoSuitableSigningKeys:
        retval = {"error": str(sys.exc_info()[1])}
    except DeSerializationNotPossible:
        retval = {"error": str(sys.exc_info()[1])}
    except:
        retval = {"error": str(sys.exc_info())}
    return retval

# command line app.
if __name__ == "__main__":

    if len(sys.argv) != 2:
        print("Usage:")
        print("verify_jws_with_jwk_url.py [JWT_FILE_PATH]")
        print("Example: verify_jws_with_jwk_url.py my.jwt")
        sys.exit(1)

    my_jwt_file = sys.argv[1]
    jwt_fh = open(my_jwt_file)

    result = verify_poet_via_url(jwt_fh.read())
    result = json.dumps(result, indent=4)
    jwt_fh.close()
    print(result)



import os
from setuptools import setup

with open(os.path.join(os.path.dirname(__file__), 'README.rst')) as readme:
    README = readme.read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(name="poetri",
      version="0.0.4",
      description="Pre-OAuth Entity Trust Reference Implementation",
      long_description="""Command line tools and libraries for signing and verifying JWTs according to https://github.com/TransparentHealth/poet.""",
      author="Alan Viars",
      author_email="sales@videntity.com",
      url="https://github.com/transparenthealth/python-poetri",
      download_url="https://github.com/transparenthealth/python-poetri/tarball/master",
      install_requires=['pyjwkest', 'requests', ],
      packages= ['poetri', 'tests'],
      scripts=['poetri/verify_jws_with_jwk.py',
               'poetri/verify_jws_with_jwk_url.py',
               'poetri/sign_poet_jwk.py',
               'poetri/generate_jwk_keypair.py',
               'poetri/generate_public_jwk.py'
               ]
      )




#!/usr/bin/env python
from jwkest.jws import JWS
from jwkest.jwk import RSAKey
import json
import sys
import time
from collections import OrderedDict

__author__ = 'Alan Viars @aviars'


def sign_poet(payload, private_key_jwk, issuer, expires=63072000):

    # Set the headers
    headers = OrderedDict()
    headers["typ"] = "JWT"
    headers["alg"] = "RS256"

    # Add/overwrite payload
    payload["iss"] = issuer
    payload["iat"] = int(time.time())

    # Set to expire (two years is default)
    payload["exp"] = payload["iat"] + expires
    rsak = RSAKey(**private_key_jwk)
    my_jws = JWS(payload, alg="RS256", typ="JWT")
    return(my_jws.sign_compact([rsak]))

# Command line app.
if __name__ == "__main__":

    if len(sys.argv) not in (4, 5):
        print("Usage:")
        print(
            "sign_poet_jwk.py [PAYLOAD_JSON_FILE] [JWK_KEYPAIR_FILE_PATH] [ISSUER] [SECONDS_UNTIL_EXPIRY]")
        print("Example: sign_poet_jwk.py mypayload.json my-certificate.pem example.org 31536000")
        print("Note: 31536000 is one year from now.")
        sys.exit(1)

    my_payload_file = sys.argv[1]
    my_jwk_file = sys.argv[2]
    issuer = sys.argv[3]
    if sys.argv == 5:
        expires = sys.argv[4]
    else:
        expires = 63072000
    my_payload_fh = open(my_payload_file)
    my_jwk_fh = open(my_jwk_file)

    # convert json to dict
    try:
        p = my_payload_fh.read()
        my_payload = json.loads(p, object_pairs_hook=OrderedDict)
    except ValueError:
        result = ["Error parsing the JSON Payload", str(sys.exc_info())]
    try:
        k = my_jwk_fh.read()
        my_jwk = json.loads(k, object_pairs_hook=OrderedDict)
    except ValueError:
        result = ["Error parsing the JWK.", str(sys.exc_info())]

    result = sign_poet(my_payload, my_jwk, issuer, expires)

    my_payload_fh.close()
    my_jwk_fh.close()

    print(result)



#!/usr/bin/env python
import unittest
import json
import os
from poetri.sign_poet_jwk import sign_poet
from poetri.verify_jws_with_jwk import verify_poet
from collections import OrderedDict

__author__ = 'Alan Viars @aviars'


class TestSignAndVerify(unittest.TestCase):

    def setUp(self):

        # Get the kid/issuer
        self.issuer = "example.com"

        # load a sample payload
        payload_fp = os.path.join(os.path.dirname(__file__), "payload.json")
        payload_fh = open(payload_fp)
        payload = payload_fh.read()
        self.payload = json.loads(payload, object_pairs_hook=OrderedDict)
        payload_fh.close()

        # load a sample keypair
        keypair_fp = os.path.join(os.path.dirname(__file__), "keypair.jwk")
        keypair_fh = open(keypair_fp)
        keypair = keypair_fh.read()
        self.keypair = json.loads(keypair, object_pairs_hook=OrderedDict)
        keypair_fh.close()

        # load a sample public key
        pubkey_fp = os.path.join(os.path.dirname(__file__), "poet.jwk")
        pubkey_fh = open(pubkey_fp)
        pubkey = pubkey_fh.read()
        self.pubkey = json.loads(pubkey, object_pairs_hook=OrderedDict)
        pubkey_fh.close()

        # A public key that doesn't match its private counterpart.
        self.bad_n = "X0vZedyiaeg_tqAcGeVzdpd_DO5QtBHpsRvDX6SKDwOytfsLZUIfR5Q"\
                     "gpz49kLCEDIjGAdg3iQ81leo2zrX5RfZ6q1n5pFpbU7VLX3ylKZ7Sug"\
                     "-ujuiNd7xmVnvdwiKyupnEnG_6XXwJDaoyMT9xXgiR4BKS3pHCoIPO0"\
                     "ktIi2BHGB1Nqb2YqKoCaeMmuZvW6EIA04_wb6wTLIXcf8jh8bt4pJ0C"\
                     "WMLqJqr524p0rEhYGl5P3BsnBDr19vM-i-_dNjAoaUT1Bc6wN_a1wFe"\
                     "baEL1C2Aia1EeF3oMAfsa_aTLA2x8NlWrjwwPyGIOsrxjmjJ6oLqvUC"\
                     "dcg1Nod9YaY9"

    def test_signing(self):

        self.signed_jws = sign_poet(self.payload,
                                    self.keypair,
                                    self.issuer)
        """Test the POET JWT signing by ensuring
           exactly two periods in output."""
        self.assertEqual(self.signed_jws.count('.'), 2)

    def test_signature_verification_good_pubkey(self):
        self.signed_jws = sign_poet(self.payload,
                                    self.keypair,
                                    self.issuer)
        self.verified_payload = verify_poet(self.signed_jws, self.pubkey)
        self.assertNotIn("error", self.verified_payload.keys())

    def test_signature_verification_good_pubkey_bad_kid(self):
        self.signed_jws = sign_poet(self.payload, self.keypair, self.issuer)
        self.pubkey['kid'] = self.pubkey['kid'] + "foo"
        self.verified_payload = verify_poet(self.signed_jws, self.pubkey)
        self.assertIn("error", self.verified_payload.keys())

    def test_signature_verification_bad_pubkey_good_kid(self):
        self.signed_jws = sign_poet(self.payload,
                                    self.keypair,
                                    self.issuer)
        # A non matching public key - modified value for n.
        self.pubkey['n'] = self.bad_n
        self.verified_payload = verify_poet(self.signed_jws, self.pubkey)
        self.assertIn("error", self.verified_payload.keys())

if __name__ == '__main__':
    unittest.main()
